/*---------------------------------------------------------------------------*\

        FILE....: COMP.CPP
        TYPE....: C++ Module
        AUTHOR..: David Rowe
        DATE....: 17/3/98

	Compression class packs and unpacks compressed speech between the DSP
	FIFOs which require one sample per 16 bit word, and the compressed
	format used by the PC which packs samples as tightly as possible.
	 
\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\

	Copyright (C) 1999 Voicetronix Pty Ltd

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

\*---------------------------------------------------------------------------*/
#include "config.h"
#include "comp.h"

/*---------------------------------------------------------------------------*\

			    class PCMCompression

\*---------------------------------------------------------------------------*/

void PCMCompression::pack(char *packedbytesout, word *wordsin, ushort nwordsin)
//	char *packedbytesout	output array of packed bytes
//      word *wordsin		input array of words
//	ushort nwordsin		number of input words
{
	int i;

	for(i=0; i<nwordsin; i++)
		packedbytesout[i] = (char)wordsin[i];
}

void PCMCompression::unpack(word *wordsout, char *packedbytesin, ushort nbytesin)
//	word *wordsout		output array of words
//       char *packedbytesin	input array of packed bytes
//	ushort nbytesin		number of input bytes
{
	int i;

	for(i=0; i<nbytesin; i++)
		wordsout[i] = (word)packedbytesin[i];
}

/*---------------------------------------------------------------------------*\

			   class LinearCompression

\*---------------------------------------------------------------------------*/

void LinearCompression::pack(char *packedbytesout, word *wordsin, ushort nwordsin)
//	char *packedbytesout	output array of packed bytes
//      word *wordsin		input array of words
//	ushort nwordsin		number of input words
{
	int i;
	word *w=(word*)packedbytesout;

	for(i=0; i<nwordsin; i++)
		w[i] = wordsin[i];
}

void LinearCompression::unpack(word *wordsout, char *packedbytesin, ushort nbytesin)
//	word *wordsout		output array of words
//      char *packedbytesin	input array of packed bytes
//	ushort nbytesin		number of input bytes
{
	int i;
	word *w=(word*)packedbytesin;

	ushort nwords = bytes2words(nbytesin);
	for(i=0; i<nwords; i++)
		wordsout[i] = w[i];
}

/*---------------------------------------------------------------------------*\

			     class ADPCMCompression

\*---------------------------------------------------------------------------*/

void ADPCMCompression::pack(char *packedbytesout, word *wordsin, ushort nwordsin)
//	char *packedbytesout	output array of packed bytes
//      word *wordsin		input array of words
//	ushort nwordsin		number of input words
{
	int i;

	ushort nbytes = words2bytes(nwordsin);
	for(i=0; i<nbytes; i++) {
		packedbytesout[i] = wordsin[2*i] << 4;
		packedbytesout[i] |= wordsin[2*i+1];
	}
}

void ADPCMCompression::unpack(word *wordsout, char *packedbytesin, ushort nbytesin)
//	word *wordsout		output array of words
//      char *packedbytesin	input array of packed bytes
//	ushort bytesin		number of input bytes
{
	int i;

	for(i=0; i<nbytesin; i++) {
		wordsout[2*i] = packedbytesin[i]>>4;
		wordsout[2*i+1] = packedbytesin[i] & 0xf;
	}
}

